
---
# Smart DOF 4 click

> PRVIH_PAR_RECENICA_SA_PRODUCT_PAGE_DA_ISPRATE_CELINU

<p align="center">
  <img src="CLICK_IMAGE_LINK" height=300px>
</p>

[click Product page](CLICK_PRODUCT_PAGE_LINK)

---


#### Click library

- **Author**        : Stefan Filipovic
- **Date**          : Aug 2024.
- **Type**          : I2C/SPI type


# Software Support

We provide a library for the Smart DOF 4 Click
as well as a demo application (example), developed using MikroElektronika
[compilers](https://www.mikroe.com/necto-studio).
The demo can run on all the main MikroElektronika [development boards](https://www.mikroe.com/development-boards).

Package can be downloaded/installed directly from *NECTO Studio Package Manager*(recommended way), downloaded from our [LibStock&trade;](https://libstock.mikroe.com) or found on [Mikroe github account](https://github.com/MikroElektronika/mikrosdk_click_v2/tree/master/clicks).

## Library Description

> This library contains API for Smart DOF 4 Click driver.

#### Standard key functions :

- `smartdof4_cfg_setup` Config Object Initialization function.
```c
void smartdof4_cfg_setup ( smartdof4_cfg_t *cfg );
```

- `smartdof4_init` Initialization function.
```c
err_t smartdof4_init ( smartdof4_t *ctx, smartdof4_cfg_t *cfg );
```

- `smartdof4_default_cfg` Click Default Configuration function.
```c
err_t smartdof4_default_cfg ( smartdof4_t *ctx );
```

#### Example key functions :

- `smartdof4_read_pid` This function reads the product ID information.
```c
err_t smartdof4_read_pid ( smartdof4_t *ctx, smartdof4_pid_t *pid );
```

- `smartdof4_feature_set` This function sets a full feature report.
```c
err_t smartdof4_feature_set ( smartdof4_t *ctx, smartdof4_feature_t *feat );
```

- `smartdof4_read_data` This function reads the accelerometer (g), gyroscope (dps), and magnetometer (uT) 3-axis data from input report. 
```c
err_t smartdof4_read_data ( smartdof4_t *ctx, smartdof4_axis_t *accel, 
                            smartdof4_axis_t *gyro, smartdof4_axis_t *mag );
```

## Example Description

> This example demonstrates the use of Smart DOF 4 click board by reading the accelerometer, gyroscope, and magnetometer data measurements.

**The demo application is composed of two sections :**

### Application Init

> Initializes the driver and performs the click default configuration enabling
accelerometer, gyroscope, and magnetometer sensors with an output data rate of 10Hz.
After that, it reads the software version, part number, and build number information.

```c

void application_init ( void )
{
    log_cfg_t log_cfg;  /**< Logger config object. */
    smartdof4_cfg_t smartdof4_cfg;  /**< Click config object. */

    /** 
     * Logger initialization.
     * Default baud rate: 115200
     * Default log level: LOG_LEVEL_DEBUG
     * @note If USB_UART_RX and USB_UART_TX 
     * are defined as HAL_PIN_NC, you will 
     * need to define them manually for log to work. 
     * See @b LOG_MAP_USB_UART macro definition for detailed explanation.
     */
    LOG_MAP_USB_UART( log_cfg );
    log_init( &logger, &log_cfg );
    log_info( &logger, " Application Init " );

    // Click initialization.
    smartdof4_cfg_setup( &smartdof4_cfg );
    SMARTDOF4_MAP_MIKROBUS( smartdof4_cfg, MIKROBUS_1 );
    err_t init_flag = smartdof4_init( &smartdof4, &smartdof4_cfg );
    if ( ( I2C_MASTER_ERROR == init_flag ) || ( SPI_MASTER_ERROR == init_flag ) )
    {
        log_error( &logger, " Communication init." );
        for ( ; ; );
    }
    
    if ( SMARTDOF4_ERROR == smartdof4_default_cfg ( &smartdof4 ) )
    {
        log_error( &logger, " Default configuration." );
        for ( ; ; );
    }

    smartdof4_pid_t pid;
    if ( SMARTDOF4_OK == smartdof4_read_pid ( &smartdof4, &pid ) )
    {
        log_printf ( &logger, " SW Version: %u.%u.%u\r\n", ( uint16_t ) pid.sw_ver_major, 
                                                           ( uint16_t ) pid.sw_ver_minor, 
                                                           ( uint16_t ) pid.sw_ver_patch );
        log_printf ( &logger, " SW Part Number: %lu\r\n", pid.sw_part_num );
        log_printf ( &logger, " SW Build Number: %lu\r\n\n", pid.sw_build_num );
    }
    
    log_info( &logger, " Application Task " );
}

```

### Application Task

> Reads the accelerometer (g), gyroscope (dps), and magnetometer (uT) measurements and displays results on the USB UART every 100ms approximately.

```c
void application_task ( void )
{
    static smartdof4_axis_t accel, gyro, mag;
    if ( SMARTDOF4_OK == smartdof4_read_data ( &smartdof4, &accel, &gyro, &mag ) )
    {
        log_printf ( &logger, " Accel X: %.3f g\r\n", accel.x );
        log_printf ( &logger, " Accel Y: %.3f g\r\n", accel.y );
        log_printf ( &logger, " Accel Z: %.3f g\r\n", accel.z );
        log_printf ( &logger, " Gyro X: %.1f dps\r\n", gyro.x );
        log_printf ( &logger, " Gyro Y: %.1f dps\r\n", gyro.y );
        log_printf ( &logger, " Gyro Z: %.1f dps\r\n", gyro.z );
        log_printf ( &logger, " Mag X: %.1f uT\r\n", mag.x );
        log_printf ( &logger, " Mag Y: %.1f uT\r\n", mag.y );
        log_printf ( &logger, " Mag Z: %.1f uT\r\n\n", mag.z );
        Delay_ms ( 100 );
    }
}
```

The full application code, and ready to use projects can be installed directly from *NECTO Studio Package Manager*(recommended way), downloaded from our [LibStock&trade;](https://libstock.mikroe.com) or found on [Mikroe github account](https://github.com/MikroElektronika/mikrosdk_click_v2/tree/master/clicks).

**Other Mikroe Libraries used in the example:**

- MikroSDK.Board
- MikroSDK.Log
- Click.SmartDOF4

**Additional notes and informations**

Depending on the development board you are using, you may need
[USB UART click](https://www.mikroe.com/usb-uart-click),
[USB UART 2 Click](https://www.mikroe.com/usb-uart-2-click) or
[RS232 Click](https://www.mikroe.com/rs232-click) to connect to your PC, for
development systems with no UART to USB interface available on the board. UART
terminal is available in all MikroElektronika
[compilers](https://shop.mikroe.com/compilers).

---
